%%% Przykłady operacji na listach: zbiory i inne.

% Podzbiór   
   
subset([],_).
subset([X|L],Set):-
      member(X,Set),!,
      subset(L,Set).
seteq(R,S):- subset(R,S),subset(S,R).

subsets([],_).
subsets([X|L],Set):-
      select(X,Set,Z),
      subsets(L,Z).
                            
% Przecięcie; not jest w zasadzie niepotrzebne, zachowano dla logiki prezentacji
intersect([],_,[]).
intersect([X|L],Set,[X|Z]):-
      member(X,Set),!,	
      intersect(L,Set,Z).
intersect([X|L],Set,Z):-
      not(member(X,Set)),
      intersect(L,Set,Z).

% Suma; not jest w zasadzie niepotrzebne, zachowano dla logiki prezentacji      
union([],Set,Set).
union([X|L],Set,Z):-
      member(X,Set),!,
      union(L,Set,Z).
union([X|L],Set,[X|Z]):-
      not(member(X,Set)),!,
      union(L,Set,Z).
  
% Różnica      
difference([],_,[]).
difference([X|L],Set,[X|Z]):-
      not(member(X,Set)),!,
      difference(L,Set,Z).
difference([_|L],Set,Z):-
      difference(L,Set,Z).

% Iloczyn kartezjañski

% definicja generatora par elementow pobieranych ze zbiorów R i S
pair(R,S,[ER,ES]) :-
	select(ER,R,_),
        select(ES,S,_).

% findall zbiera wszystkie pary na liste RS
prod(R,S,RS) :-
	findall(P,pair(R,S,P),RS),
	writelist(RS).

%%% Inne

% Wypisywanie listy    
writelist([]):-!,nl.
writelist([X|L]):-
       write(X),nl,
       writelist(L).

% Dodawanie elementu (bez sprawdzania czy należy)
add(X,L,[X|L]).


% Usuwanie elementu (niedeterministyczne; równoważne select).
del(X,[X|L],L).
del(X,[Y|L],[Y|L1]):-
       del(X,L,L1).

delall(_,[],[]).
delall(X,[H|L],[H|LL]):- X \== H,!,delall(X,L,LL).
delall(X,[X|L],LL):- delall(X,L,LL).
%delall(X,[Y|L],[Y|L1]):-
%       del(X,L,L1).



% Dodawanie elemntu do listy; niedeterministyczne
insert(X,L,LX):-
       del(X,LX,L).   
% Sprawdzanie podlisty
sublist(S,L):-
       append(_,L2,L),
       append(S,_,L2).   

% Odwracanie
reverse([],[]).
reverse([X|L],R):-
       reverse(L,RL),
       append(RL,[X],R).

% Odwracanie z akumulatorem
inverse(L,R):-
       do([],L,R).
    do(L,[],L):-!.
    do(L,[X|T],S):- 
       write(L),write('   '),write([X|T]),nl,
       do([X|L],T,S).

% Suma iteracyjna

itersum([X|T],S,U):-
   put(X,S,XS),
   itersum(T,XS,U).
itersum([],U,U).
put(X,S,S):- member(X,S),!.
put(X,S,[X|S]).

put_last(X,[],[X]).
put_last(X,[H|T],[H|L]):- put_last(X,T,L). 

podzb([],_):-!.
podzb([H|T],S):-
   member(H,S),
   podzb(T,S).

rownzb(S,R):- podzb(S,R), podzb(R,S),!.

dod(X,S,[X|S]).

sum_zbior([],S,S).
sum_zbior([H|T],S,ST):- member(H,S),!, sum_zbior(T,S,ST).
sum_zbior([H|T],S,[H|ST]):- \+member(H,S),!, sum_zbior(T,S,ST).

przec_zbior([],_,[]).
przec_zbior([H|T],S,ST):- \+member(H,S),!, przec_zbior(T,S,ST).
przec_zbior([H|T],S,[H|ST]):- member(H,S),!, przec_zbior(T,S,ST).





